@@ -1,43 +1,44 @@
 #' Test one or more conditions on HTML elements
-#' 
-#' @description 
+#'
+#' @description
 #' `elem_expect()` waits for a set of conditions to return TRUE. If, after a
 #' certain period of time (by default 4 seconds), this does not happen, an
 #' informative error is thrown. Otherwise, the original element is returned.
-#' 
+#'
 #' `elem_wait_until()` does the same, but returns a logical value (whether or
 #' not the test passed), allowing you to handle the failure case explicitly.
-#' 
+#'
 #' @param x A `selenider_element`/`selenider_elements` object, or a condition.
 #' @param ... <[`dynamic-dots`][rlang::dyn-dots]> Function calls or functions
 #'   that must return a logical value. If multiple conditions are given, they
 #'   must all be `TRUE` for the test to pass.
-#' @param testthat Whether to treat the expectation as a `testthat` test. You 
-#'   *do not* need to explicitly provide this most of the time, since by default,
-#'   we can use [testthat::is_testing()] to figure out whether `elem_expect()` is
-#'   being called from within a `testthat` test.
+#' @param testthat Whether to treat the expectation as a `testthat` test. You
+#'   *do not* need to explicitly provide this most of the time, since by
+#'   default, we can use [testthat::is_testing()] to figure out whether
+#'   `elem_expect()` is being called from within a `testthat` test.
 #' @param timeout The number of seconds to wait for a condition to pass. If not
 #'   specified, the timeout used for `x` will be used, or the timeout of the
 #'   local session if an element is not given.
-#' 
+#'
 #' @details
 #' # Conditions
 #' Conditions can be supplied as functions or calls.
-#' 
+#'
 #' Functions allow you to use unary conditions without formatting them as a
-#' call (e.g. `is_present` rather than `is_present()`). It also allows you to make
-#' use of R's [anonymous function syntax][base::function] to quickly create
-#' custom conditions. `x` will be supplied as the first argument of this function.
-#' 
-#' Function calls allow you to use conditions that take multiple arguments (e.g.
-#' `has_text()`) without the use of an intermediate function. The call will
-#' be modified so that `x` is the first argument to the function call. For 
+#' call (e.g. `is_present` rather than `is_present()`). It also allows you to
+#' make use of R's [anonymous function syntax][base::function] to quickly
+#' create custom conditions. `x` will be supplied as the first argument of this
+#' function.
+#'
+#' Function calls allow you to use conditions that take multiple arguments
+#' (e.g. `has_text()`) without the use of an intermediate function. The call
+#' will be modified so that `x` is the first argument to the function call. For
 #' example, `has_text("a")` will be modified to become: `has_text(x, "a")`.
 #'
 #' The and (`&&`), or (`||`) and not (`!`) functions can be used on both types
 #' of conditions. If more than one condition are given in `...`, they are
 #' combined using `&&`.
-#' 
+#'
 #' # Custom conditions
 #' Any function which takes a selenider element or element collection as its
 #' first argument, and returns a logical value, can be used as a condition.
@@ -56,31 +57,31 @@
 #'
 #' These two features allow you to use functions like [elem_text()] to access
 #' properties of an element, without needing to worry about the errors that
-#' they throw or the timeouts that they use. See Examples for a few examples of a
-#' custom condition.
+#' they throw or the timeouts that they use. See Examples for a few examples of
+#' a custom condition.
 #'
 #' These custom conditions can also be used with [elem_filter()] and
 #' [elem_find()].
 #'
-#' @returns 
-#' `elem_expect()` invisibly returns the element(s) `x`, or `NULL` if an element or
-#' collection of elements was not given in `x`.
-#' 
+#' @returns
+#' `elem_expect()` invisibly returns the element(s) `x`, or `NULL` if an
+#' element or collection of elements was not given in `x`.
+#'
 #' `elem_wait_for()` returns a boolean flag: TRUE if the test passes, FALSE
 #' otherwise.
-#' 
-#' @seealso 
+#'
+#' @seealso
 #' * [is_present()] and other conditions for predicates for HTML elements.
 #'   (If you scroll down to the *See also* section, you will find the rest).
-#' * [elem_expect_all()] and [elem_wait_until_all()] for an easy way to test a single
-#'   condition on multiple elements.
+#' * [elem_expect_all()] and [elem_wait_until_all()] for an easy way to test a
+#'   single condition on multiple elements.
 #' * [elem_filter()] and [elem_find()] to use conditions to filter elements.
-#' 
+#'
 #' @examplesIf selenider::selenider_available(online = FALSE)
 #' html <- "
 #' <div class='class1'>
 #' <button id='disabled-button' disabled>Disabled</button>
-#' <p style='visibility:hidden;'>Example text</p>
+#' <p>Example text</p>
 #' <button id='enabled-button'>Enabled</button>
 #' </div>
 #'
@@ -94,10 +95,10 @@
 #'
 #' s("#enabled-button") |>
 #'   elem_expect(is_visible, is_enabled)
-#' 
+#'
 #' s("#disabled-button") |>
 #'   elem_expect(is_disabled)
-#' 
+#'
 #' # Error: element is visible but not enabled
 #' s("#disabled-button") |>
 #'   elem_expect(is_visible, is_enabled, timeout = 0.5) |>
@@ -118,19 +119,21 @@
 #' # Note that elem_click() will do this automatically
 #'
 #' s("p") |>
-#'   elem_expect(is_hidden, has_exact_text("Example text"))
+#'   elem_expect(is_visible, has_exact_text("Example text"))
 #'
 #' # Or use an anonymous function
 #' s("p") |>
 #'   elem_expect(\(elem) identical(elem_text(elem), "Example text"))
 #'
-#' # If your conditions are not specific to an element, you can omit the `x` argument
+#' # If your conditions are not specific to an element, you can omit the `x`
+#' # argument
 #' elem_1 <- s(".class1")
 #' elem_2 <- s(".class2")
 #'
 #' elem_expect(is_present(elem_1) || is_present(elem_2))
-#' 
-#' # We can now use the conditions on their own to figure out which element exists
+#'
+#' # We can now use the conditions on their own to figure out which element
+#' # exists
 #' if (is_present(elem_1)) {
 #'   print("Element 1 is visible")
 #' } else {
@@ -152,19 +155,20 @@
 #'       elem_text() |>
 #'       grepl(pattern = "Example .*")
 #'   )
-#' 
+#'
 #' # Or create a function, to reuse the condition multiple times
 #' text_contains <- function(x, pattern) {
 #'   text <- elem_text(x)
-#'   
+#'
 #'   grepl(pattern, text)
 #' }
 #'
 #' s("p") |>
 #'   elem_expect(text_contains("Example *"))
 #'
-#' # If we want to continue on error, we need to use the "expect_error_continue" class
-#' # This involves making a custom error object
+#' # If we want to continue on error, we need to use the
+#' # "expect_error_continue" class.
+#' # This involves making a custom error object.
 #' error_condition <- function() {
 #'   my_condition <- list(message = "Custom error!")
 #'   class(my_condition) <- c("expect_error_continue", "error", "condition")
@@ -187,7 +191,7 @@
 #' # Clean up all connections and invalidate default chromote object
 #' selenider_cleanup()
 #' }
-#' 
+#'
 #' @export
 elem_expect <- function(x, ..., testthat = NULL, timeout = NULL) {
   x <- enquo(x)
@@ -201,7 +205,7 @@ elem_expect <- function(x, ..., testthat = NULL, timeout = NULL) {
   } else if (testthat) {
     check_installed("testthat", reason = "for `elem_expect(testthat = TRUE)`.")
   }
-  
+
   result <- eval_conditions(x, dots, timeout)
   timeout <- result$timeout
   calls <- result$calls
@@ -212,7 +216,13 @@ elem_expect <- function(x, ..., testthat = NULL, timeout = NULL) {
   if (!isTRUE(res)) {
     n <- res$n
     val <- res$val
-    x_res <- if (inherits(x_res, c("selenider_element", "selenider_elements"))) x_res else NULL
+    classes <- c("selenider_element", "selenider_elements")
+    x_res <- if (inherits_any(x_res, classes)) {
+      x_res
+    } else {
+      NULL
+    }
+
     call <- calls[[n]]
     expr <- exprs[[n]]
 
@@ -229,7 +239,8 @@ elem_expect <- function(x, ..., testthat = NULL, timeout = NULL) {
   }
 
   if (testthat) {
-    # We have already checked that testthat is installed if `testthat` is `TRUE`.
+    # We have already checked that testthat is installed if `testthat` is
+    # `TRUE`.
     testthat::succeed()
   }
 
@@ -268,12 +279,14 @@ elem_wait_until <- function(x, ..., timeout = NULL) {
 #'   if this is not `TRUE`).
 #' @param timeout The number of seconds of timeout that was used.
 #' @param original_env The environment in which the conditions wer evaluated.
-#' @param testthat Whether to create a testthat test failure instead of an error.
+#' @param testthat Whether to create a testthat test failure instead of an
+#'   error.
 #' @param call_name The name of the condition.
 #' @param negated_call_name The negated name of the condition.
-#' @param call_env The environment of the [elem_expect()] call, to throw the error in.
-#' @param x_name Used by [elem_expect_all()] to specify that a specific argument failed
-#'   (e.g. "x[[1]]").
+#' @param call_env The environment of the [elem_expect()] call, to throw the
+#'   error in.
+#' @param x_name Used by [elem_expect_all()] to specify that a specific
+#'   argument failed (e.g. "x[[1]]").
 #'
 #' @noRd
 diagnose_condition <- function(x,
@@ -287,7 +300,7 @@ diagnose_condition <- function(x,
                                call_env = caller_env(),
                                x_name = "x") {
   call_name <- if (is_call_simple(call)) call_name(call) else ""
-  
+
   expr_print <- paste0(
     expr_deparse(quo_squash(original_expr)), collapse = "\n"
   )
@@ -311,7 +324,10 @@ diagnose_condition <- function(x,
     } else {
       condition <- c(
         condition,
-        "i" = "The condition returned {.obj_type_friendly {result}} instead of {.val {TRUE}}."
+        "i" = paste0(
+          "The condition returned {.obj_type_friendly {result}} instead of ",
+          "{.val {TRUE}}."
+        )
       )
 
       if (is_function(result) && n == 1) {
@@ -337,10 +353,17 @@ diagnose_condition <- function(x,
   condition <- condition_result$condition
   call_name <- condition_result$call_name
   negated_call_name <- condition_result$negated_call_name
-  
+
   if (testthat) {
-    # We have already checked that testthat is installed if `testthat` is `TRUE`.
-    elem_expect_fail(condition, parent = parent, call = call_env, x = x, x_name = x_name)
+    # We have already checked that testthat is installed if `testthat` is
+    # `TRUE`.
+    elem_expect_fail(
+      condition,
+      parent = parent,
+      call = call_env,
+      x = x,
+      x_name = x_name
+    )
   } else {
     stop_expect_error(condition, parent = parent, call = call_env)
   }
@@ -352,241 +375,84 @@ get_call_arg <- function(call, name) {
   args[[name]]
 }
 
-diagnose_condition_call <- function(condition, x, call, call_name, original_env, negated_call_name = NULL) {
+diagnose_condition_call <- function(condition,
+                                    x,
+                                    call,
+                                    call_name,
+                                    original_env,
+                                    negated_call_name = NULL) {
   call_name <- switch(call_name,
     is_in_dom = "is in the DOM",
     has_name = "has tag name",
     gsub("_", " ", call_name, fixed = TRUE)
   )
 
-  if (call_name %in% c("(", "!")) {
-    invert <- FALSE
-    inner_call_name <- call_name
-    new_call <- call
-    while (inner_call_name %in% c("(", "!")) {
-      if (inner_call_name == "!") {
-        invert <- !invert
-      }
-      new_call <- call_args(new_call)[[1]]
-      inner_call_name <- if (is_call_simple(new_call)) call_name(new_call) else ""
-    }
-
-    inner_call_name <- switch(inner_call_name,
-      is_in_dom = "is in the DOM",
-      has_name = "has tag name",
-      gsub("_", " ", inner_call_name, fixed = TRUE)
-    )
-    
-    if (!invert) {
-      return(diagnose_condition_call(
-        condition,
-        x,
-        call = new_call,
-        call_name = inner_call_name,
-        original_env = original_env
-      ))
-    }
-
-    new_call_name <- negate_call_name(inner_call_name)
-
-    if (!is.null(x)) {
-      return(diagnose_condition_call(
-        condition,
-        x,
-        call = new_call,
-        call_name = new_call_name,
-        negated_call_name = inner_call_name,
-        original_env = original_env
-      ))
-    }
-  } else if (call_name %in% condition_dependencies$none) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    condition <- c(
+  condition <- if (call_name %in% c("(", "!")) {
+    return(diagnose_condition_inverse(
       condition,
-      "i" = "{.arg {x_name}} {negated_call_name}."
-    )
+      x = x,
+      call_name = call_name,
+      call = call,
+      original_env = original_env
+    ))
+  } else if (call_name %in% condition_dependencies$none) {
+    diagnose_condition_none(condition, call_name, negated_call_name)
   } else if (call_name %in% condition_dependencies$name) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    expected_name <- eval_tidy(get_call_arg(call, "name"), env = original_env)
-    actual_name <- get_or_null(x, elem_name, timeout = 0)
-
-    if (is.null(actual_name)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(expected_name), ".")
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(expected_name), "."),
-        "i" = paste0("Actual tag name: ", format_value(actual_name), ".")
-      )
-    }
+    diagnose_condition_name(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
+    )
   } else if (call_name %in% condition_dependencies$text) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    target_text <- eval_tidy(get_call_arg(call, "text"), env = original_env)
-    actual_text <- get_or_null(x, elem_text, timeout = 0)
-
-    if (is.null(actual_text)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(target_text), ".")
-      )
-    } else {
-      actual_text <- tryCatch(elem_text(x, timeout = 0), error = function(e) NULL)
-
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(target_text), "."),
-        "i" = paste0("Actual text: ", format_value(actual_text), ".")
-      )
-    }
+    diagnose_condition_text(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
+    )
   } else if (call_name %in% condition_dependencies$attribute) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    negated_call_name <- switch(
-      negated_call_name,
-      "has attr" = "is",
-      "attr contains" = "contains",
-      "does not have attr" = "is not",
-      "attr does not contain" = "does not contain"
+    diagnose_condition_attribute(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
     )
-
-    name <- eval_tidy(get_call_arg(call, "name"), env = original_env)
-    expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
-    actual_value <- get_or_null(x, elem_attr, name, timeout = 0)
-    
-    if (is.null(actual_value)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}}'s ", format_value(name), " attribute {negated_call_name} ", format_value(expected_value), ".")
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}}'s ", format_value(name), " attribute {negated_call_name} ", format_value(expected_value), "."),
-        "i" = paste0("Actual value: ", format_value(actual_value), ".")
-      )
-    }
   } else if (call_name %in% condition_dependencies$value) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
-    expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
-    actual_value <- get_or_null(x, elem_value, timeout = 0)
-    
-    if (is.null(actual_value)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(expected_value), ".")
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} {negated_call_name} ", format_value(expected_value), "."),
-        "i" = paste0("Actual value: ", format_value(actual_value), ".")
-      )
-    }
+    diagnose_condition_value(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
+    )
   } else if (call_name %in% condition_dependencies$css) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    negated_call_name <- switch(
-      negated_call_name,
-      "has css property" = "is",
-      "does not have css property" = "is not"
+    diagnose_condition_css(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
     )
-
-    name <- eval_tidy(get_call_arg(call, "property"), env = original_env)
-    expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
-    actual_value <- get_or_null(x, elem_css_property, name, timeout = 0)
-    
-    if (is.null(actual_value)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}}'s ", format_value(name), " CSS property {negated_call_name} ", format_value(expected_value), ".")
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}}'s ", format_value(name), " CSS property {negated_call_name} ", format_value(expected_value), "."),
-        "i" = paste0("Actual value: ", format_value(actual_value), ".")
-      )
-    }
   } else if (call_name %in% condition_dependencies$length) {
-    if (is.null(negated_call_name)) {
-      negated_call_name <- negate_call_name(call_name)
-    }
-
-    value <- eval_tidy(get_call_arg(call, "n"), env = original_env)
-    elements <- if(value == 1) "element" else "elements"
-    actual_length <- get_or_null(x, elem_size, timeout = 0)
-
-    cond <- switch(
-      negated_call_name,
-      "has length" = paste("contains", value, elements),
-      "does not have length" = paste("does not contain", value, elements),
-      "has at least" = paste("contains at least", value, elements),
-      "does not have at least" = paste("contains less than", value, elements),
+    diagnose_condition_length(
+      condition,
+      x = x,
+      call_name = call_name,
+      negated_call_name = negated_call_name,
+      call = call,
+      original_env = original_env
     )
-
-    if (is.null(actual_length)) {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} ", cond, ".")
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg {x_name}} ", cond, "."),
-        "i" = paste0("Actual number of elements: {.val ", actual_length, "}.")
-      )
-    }
-  } else if (inherits(x, "selenider_element")) {
-    if (is_present(x)) {
-      condition <- c(
-        condition,
-        "i" = "{.arg {x_name}} exists, but the condition still failed."
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = "{.arg {x_name}} does not exist, which may have caused the condition to fail."
-      )
-    }
-  } else if (inherits(x, "selenider_elements")) {
-    l <- tryCatch(elem_size(x, timeout = 0), selenider_error_absent_parent = function(e) NULL)
-
-    if (is.null(l)) {
-      condition <- c(
-        condition,
-        "i" = "{.arg x}'s parent element does not exist, which may have caused the condition to fail."
-      )
-    } else if (l == 0L) {
-      condition <- c(
-        condition,
-        "i" = "{.arg x} contains no elements, which may have caused the condition to fail."
-      )
-    } else {
-      condition <- c(
-        condition,
-        "i" = paste0("{.arg x} contains {.val {", l, "}} element{?s}, but the condition still failed.")
-      )
-    }
+  } else {
+    diagnose_condition_general(condition, x)
   }
 
   list(
@@ -596,16 +462,388 @@ diagnose_condition_call <- function(condition, x, call, call_name, original_env,
   )
 }
 
+diagnose_condition_inverse <- function(condition,
+                                       x,
+                                       call_name,
+                                       call,
+                                       original_env) {
+  invert <- FALSE
+  inner_call_name <- call_name
+  new_call <- call
+  while (inner_call_name %in% c("(", "!")) {
+    if (inner_call_name == "!") {
+      invert <- !invert
+    }
+
+    new_call <- call_args(new_call)[[1]]
+
+    inner_call_name <- if (is_call_simple(new_call)) {
+      call_name(new_call)
+    } else {
+      ""
+    }
+  }
+
+  inner_call_name <- switch(inner_call_name,
+    is_in_dom = "is in the DOM",
+    has_name = "has tag name",
+    gsub("_", " ", inner_call_name, fixed = TRUE)
+  )
+
+  if (!invert) {
+    return(diagnose_condition_call(
+      condition,
+      x,
+      call = new_call,
+      call_name = inner_call_name,
+      original_env = original_env
+    ))
+  }
+
+  new_call_name <- negate_call_name(inner_call_name)
+
+  diagnose_condition_call(
+    condition,
+    x,
+    call = new_call,
+    call_name = new_call_name,
+    negated_call_name = inner_call_name,
+    original_env = original_env
+  )
+}
+
+diagnose_condition_none <- function(condition, call_name, negated_call_name) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  c(
+    condition,
+    "i" = paste0("{.arg {x_name}} ", negated_call_name, ".")
+  )
+}
+
+diagnose_condition_name <- function(condition,
+                                    x,
+                                    call_name,
+                                    call,
+                                    negated_call_name,
+                                    original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  expected_name <- eval_tidy(get_call_arg(call, "name"), env = original_env)
+  actual_name <- get_or_null(x, elem_name, timeout = 0)
+
+  if (is.null(actual_name) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(expected_name),
+        "."
+      )
+    )
+  } else {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(expected_name),
+        "."
+      ),
+      "i" = paste0("Actual tag name: ", format_value(actual_name), ".")
+    )
+  }
+}
+
+diagnose_condition_text <- function(condition,
+                                    x,
+                                    call_name,
+                                    negated_call_name,
+                                    call,
+                                    original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  target_text <- eval_tidy(get_call_arg(call, "text"), env = original_env)
+  actual_text <- get_or_null(x, elem_text, timeout = 0)
+
+  if (is.null(actual_text) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(target_text),
+        "."
+      )
+    )
+  } else {
+    actual_text <- tryCatch(elem_text(x, timeout = 0), error = function(e) NULL)
+
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(target_text),
+        "."
+      ),
+      "i" = paste0("Actual text: ", format_value(actual_text), ".")
+    )
+  }
+}
+
+diagnose_condition_attribute <- function(condition,
+                                         x,
+                                         call_name,
+                                         negated_call_name,
+                                         call,
+                                         original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  negated_call_name <- switch(
+    negated_call_name,
+    "has attr" = "is",
+    "attr contains" = "contains",
+    "does not have attr" = "is not",
+    "attr does not contain" = "does not contain"
+  )
+
+  name <- eval_tidy(get_call_arg(call, "name"), env = original_env)
+  expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
+  actual_value <- get_or_null(x, elem_attr, name, timeout = 0)
+
+  if (is.null(actual_value) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}}'s ",
+        format_value(name),
+        " attribute ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      )
+    )
+  } else {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}}'s ",
+        format_value(name),
+        " attribute ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      ),
+      "i" = paste0("Actual value: ", format_value(actual_value), ".")
+    )
+  }
+}
+
+diagnose_condition_value <- function(condition,
+                                     x,
+                                     call_name,
+                                     negated_call_name,
+                                     call,
+                                     original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
+  expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
+  actual_value <- get_or_null(x, elem_value, timeout = 0)
+
+  if (is.null(actual_value) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      )
+    )
+  } else {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}} ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      ),
+      "i" = paste0("Actual value: ", format_value(actual_value), ".")
+    )
+  }
+}
+
+diagnose_condition_css <- function(condition,
+                                   x,
+                                   call_name,
+                                   negated_call_name,
+                                   call,
+                                   original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  negated_call_name <- switch(
+    negated_call_name,
+    "has css property" = "is",
+    "does not have css property" = "is not"
+  )
+
+  name <- eval_tidy(get_call_arg(call, "property"), env = original_env)
+  expected_value <- eval_tidy(get_call_arg(call, "value"), env = original_env)
+  actual_value <- get_or_null(x, elem_css_property, name, timeout = 0)
+
+  if (is.null(actual_value) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}}'s ",
+        format_value(name),
+        " CSS property ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      )
+    )
+  } else {
+    c(
+      condition,
+      "i" = paste0(
+        "{.arg {x_name}}'s ",
+        format_value(name),
+        " CSS property ",
+        negated_call_name,
+        " ",
+        format_value(expected_value),
+        "."
+      ),
+      "i" = paste0("Actual value: ", format_value(actual_value), ".")
+    )
+  }
+}
+
+diagnose_condition_length <- function(condition,
+                                      x,
+                                      call_name,
+                                      negated_call_name,
+                                      call,
+                                      original_env) {
+  if (is.null(negated_call_name)) {
+    negated_call_name <- negate_call_name(call_name)
+  }
+
+  value <- eval_tidy(get_call_arg(call, "n"), env = original_env)
+  elements <- if (value == 1) "element" else "elements"
+  actual_length <- get_or_null(x, elem_size, timeout = 0)
+
+  cond <- switch(
+    negated_call_name,
+    "has length" = paste("contains", value, elements),
+    "does not have length" = paste("does not contain", value, elements),
+    "has at least" = paste("contains at least", value, elements),
+    "does not have at least" = paste("contains less than", value, elements),
+  )
+
+  if (is.null(actual_length) && !is_in_dom(x)) {
+    c(
+      condition,
+      "i" = paste0("{.arg {x_name}} ", cond, ".")
+    )
+  } else {
+    c(
+      condition,
+      "i" = paste0("{.arg {x_name}} ", cond, "."),
+      "i" = paste0("Actual number of elements: {.val {", actual_length, "}}.")
+    )
+  }
+}
+
+diagnose_condition_general <- function(condition,
+                                       x) {
+  if (inherits(x, "selenider_element")) {
+    if (is_present(x)) {
+      c(
+        condition,
+        "i" = "{.arg {x_name}} exists, but the condition still failed."
+      )
+    } else {
+      c(
+        condition,
+        "i" = paste0(
+          "{.arg {x_name}} does not exist, which may have caused the ",
+          "condition to fail."
+        )
+      )
+    }
+  } else if (inherits(x, "selenider_elements")) {
+    l <- tryCatch(
+      elem_size(x, timeout = 0),
+      selenider_error_absent_parent = function(e) NULL
+    )
+
+    if (is.null(l)) {
+      c(
+        condition,
+        "i" = paste0(
+          "{.arg {x_name}}'s parent element does not exist, which may have ",
+          "caused the condition to fail."
+        )
+      )
+    } else if (l == 0L) {
+      c(
+        condition,
+        "i" = paste0(
+          "{.arg {x_name}} contains no elements, which may have caused the ",
+          "condition to fail."
+        )
+      )
+    } else {
+      c(
+        condition,
+        "i" = paste0(
+          "{.arg {x_name}} contains {.val {",
+          l,
+          "}} element{?s}, but the condition still failed."
+        )
+      )
+    }
+  }
+}
+
 negate_call_name <- function(x) {
-  # Replace e.g. !is present with is not present, but cancel out double negatives
+  # Replace e.g. !is present with is not present, but cancel out double
+  # negatives
   switch(x,
     "is present" = "is not present",
     "is in the DOM" = "is not in the DOM",
-    "is missing" = "is present", 
+    "is missing" = "is present",
     "is absent" = "is in the DOM",
     "is visible" = "is not visible",
     "is displayed" = "is not displayed",
-    "is hidden" = "is displayed", 
+    "is hidden" = "is displayed",
     "is invisible" = "is visible",
     "is enabled" = "is not enabled",
     "is disabled" = "is enabled",
