@@ -8,24 +8,37 @@
 #' @param call The environment of [elem_expect()], to throw the error in.
 #' @param x The element on which the conditions failed.
 #' @param x_name The name describing `x`.
-#' @param env The environment in which to evaluate the `condition` bullets using glue.
+#' @param env The environment in which to evaluate the `condition` bullets
+#' using glue.
 #'
 #' @noRd
-elem_expect_fail <- function(condition, parent, call, x, x_name, env = rlang::caller_env()) {
+elem_expect_fail <- function(condition,
+                             parent,
+                             call,
+                             x,
+                             x_name,
+                             env = rlang::caller_env()) {
   condition_text <- cli::format_error(condition, .envir = env)
 
   if (!is.null(parent)) {
     # Capture the "Caused by error in fn()" message.
     error_container <- try_fetch(abort("", parent = parent), error = identity)
-    parent_text <- cnd_message(error_container) # Contains a newline at the start
+
+    # Contains a newline at the start
+    parent_text <- cnd_message(error_container)
 
     condition_text <- paste0(condition_text, parent_text)
   }
 
   if (!is.null(x)) {
+    formatted <- if (inherits(x, "selenider_element")) {
+      format_lazy_selenider_element(x)
+    } else {
+      format_lazy_selenider_elements(x)
+    }
     object_text <- paste0(
       "Where `", x_name, "` is:\n",
-      paste(format(x), collapse = "\n"),
+      paste(formatted, collapse = "\n"),
       "\n"
     )
 
