@@ -2,7 +2,7 @@ selenider_test_session <- function(x, .env = rlang::caller_env()) {
   session <- Sys.getenv("SELENIDER_SESSION", "chromote")
   browser <- Sys.getenv("SELENIDER_BROWSER", "chrome")
   docker <- as.logical(Sys.getenv("SELENIDER_DOCKER", "FALSE"))
-  port <- as.integer(Sys.getenv("SELENIDER_PORT", "4567"))
+  port <- as.integer(Sys.getenv("SELENIDER_PORT", "4444"))
 
   skip_if_selenider_unavailable(session)
 
@@ -18,16 +18,30 @@ selenider_test_session <- function(x, .env = rlang::caller_env()) {
 
     result <- selenider_session(session, browser = browser, view = view, .env = .env)
 
-    withr::defer({
-      # Delete the Crashpad folder if it exists
-      unlink(file.path(tempdir(), "Crashpad"), recursive = TRUE)
-    }, envir = .env)
-  } else if (docker) {
+    withr::defer(
+      {
+        # Delete the Crashpad folder if it exists
+        unlink(file.path(tempdir(), "Crashpad"), recursive = TRUE)
+      },
+      envir = .env
+    )
+  } else if (docker && session == "selenium") {
     client <- create_selenium_client(browser, port = port)
 
     result <- selenider_session(driver = client, .env = .env)
-  } else {
+  } else if (session == "selenium") {
     result <- selenider_session(session, browser = browser, .env = .env)
+  } else if (docker && session == "rselenium") {
+    client <- create_rselenium_client(browser, port = port)
+
+    result <- selenider_session(driver = list(client = client), .env = .env)
+  } else {
+    result <- selenider_session(
+      "rselenium",
+      browser = browser,
+      selenium_manager = FALSE,
+      .env = .env
+    )
   }
 
   result
